// #autoload
// #name = Kerb's T2 Scripts
// #category = Kerb's Scripts
// #version = 2.1.2
// #date = November 4, 2002
// #status = beta
// #author = David Gausebeck
// #warrior = Kerb
// #email = gausebec@paypal.com
// #include = scripts/Kerb/loadout.cs
// #include = support/vehicle_callbacks.cs 0.0.5
// #config = kScriptOptions

// (c) 2002 David Gausebeck
// gausebec@paypal.com

// Credit to (ideas/code stolen from :)
// UberGuy, Halide, VeCTeR, Crunchy, DeLukas, Misanthrope, Writer and other
// contributors to support.vl2, and the authors of the T2 game engine scripts.

//*****************************************************************************
// SETUP
//*****************************************************************************

// set default preferences
$kPref::rangeReticles = 1;
$kPref::vehicleJoy = 0;
$kPref::maxThrow = 0;
$kPref::jumpJet = 0;
$kPref::pilotBombSight = 0;
$kPref::deployableIndicator = 0;
$kPref::autoMissile = 0;
$kPref::autoFlareHoming = 0;
$kPref::autoFlareLock = 0;
$kPref::zoomToggle = 0;
$kPref::wheelZoom = 0;
$kPref::wheelZoomSens = 2.0;
$kPref::useDefaultZoomFOV = 0;
$kPref::defaultZoomFOV = 30;

// make sure we have the vehicle controls ActionMap
if(!isObject(kScriptVehicleMap))
	new ActionMap(kScriptVehicleMap);

// load real prefs and vehicle controls if available
exec("prefs/kPrefs.cs");

// exec gui scripts
if(!isObject(kScriptMortarReticle) || !isObject(kScriptGrenadeReticle))
	exec("scripts/Kerb/kScriptReticles.cs");
if(!isObject(kScriptOptions) || !isObject(kScriptOptionsRemapDlg))
	exec("scripts/Kerb/kScriptOptions.cs");

// register callbacks
callback.add(PlayerDeath, kScriptUnzoom);
callback.add(PlayerSpawn, kScriptUnzoom);
callback.add(PlayerSpawn, kScriptPrimeGrenade);
callback.add(GrenadeReceived, kScriptPrimeGrenade);
callback.add(GrenadeUsed, kScriptPrimeGrenade);
callback.add(onPilotingAirVehicle, kScriptVehicleAirSettings);
callback.add(onPilotingAirVehicle, kScriptVehicleStart);
callback.add(onDrivingLandVehicle, kScriptVehicleGroundSettings);
callback.add(onDrivingLandVehicle, kScriptVehicleStart);
callback.add(onBomberPilot, kScriptStartBombSight);
callback.add(onVehicleDismount, kScriptVehicleEnd);
callback.add(onVehicleDismount, kScriptDeactivateReticles);
callback.add(onVehicleDismount, kScriptEndBombSight);
callback.add(onVehicleDismount, kScriptCheckDeployable);
addMessageCallback('MsgDeployFailed', kScriptCheckDeployable);

// set initial state
$kScript::pilotingVehicle = 0;
$kScript::bindsListed = 0;
	
//*****************************************************************************
// OVERRIDES
//*****************************************************************************

package kScriptOverrides {

//-----------------------------------------------------------------------------

function OptionsDlg::onWake(%this) {
	if(!$kScript::bindsListed) {
		$RemapName[$RemapCount]="Rangefinder";
		$RemapCmd[$RemapCount]="kScriptRangefinder";
		$RemapCount++;
		$RemapName[$RemapCount]="Eject and Jet";
		$RemapCmd[$RemapCount]="kScriptJumpJet";
		$RemapCount++;
		$kScript::bindsListed = 1;
	}
	parent::onWake(%this);
}

//-----------------------------------------------------------------------------

function PlayGui::onWake(%this) {
	parent::onWake(%this);

	%this.add(kScriptMortarReticle);
	%this.add(kScriptGrenadeReticle);
}

//-----------------------------------------------------------------------------

function mouseJet(%keydown)
{
	parent::mouseJet(%keydown);

	if(!$kPref::jumpJet)
		return;

	if(%keydown && !($mvTriggerCount2 & 1) && getCurrentPosition() !$= "Pilot") {
		jump(1);
		schedule(100, 0, jump, 0);
	}
}

//-----------------------------------------------------------------------------

function throwGrenade(%keydown)
{
	if(!$kPref::maxThrow) {
		parent::throwGrenade(%keydown);
		return;
	}
		
	if(%keydown) {
		if(!$kScript::blockGrenade) {
			// safe to throw a normal grenade
			commandToServer('endThrowCount');
			if($mvTriggerCount4 & 1)
				$mvTriggerCount4++;
			// don't allow another throw for .8 seconds, because it wouldn't
			// go off anyway and it will mess up the priming
			$kScript::blockGrenade = 1;
			schedule(800, 0, kScriptUnblockGrenade);
		}
		else {
			// if it's not safe to throw a normal gren, try to throw a flare.
			// this allows you to throw flares as fast as you want, even though
			// normal grens are limited to 1/.8 secs.
			use("FlareGrenade");
		}
	}
}

//-----------------------------------------------------------------------------

function toggleZoom(%keydown)
{
	if($kPref::zoomToggle && !%keydown)
		return;

	if($kPref::zoomToggle && $ZoomON)
		%keydown = 0;

	if(!$ZoomON && %keydown && $kPref::useDefaultZoomFOV)
		$pref::player::currentFOV = $kPref::defaultZoomFOV;

	parent::toggleZoom(%keydown);

	// if we just zoomed in, set mousewheel to control zoom level
	if($ZoomON && $kPref::wheelZoom) {
		if(!isObject(kScriptWheelZoomMap)) {
			new ActionMap(kScriptWheelZoomMap);
			kScriptWheelZoomMap.bind(mouse0, zaxis, kScriptZoomAxis);
		}
		kScriptWheelZoomMap.push();
	}
	
	// if we just zoomed out, remove mousewheel zoom control (even if
	// prefs aren't set for it, since the pref may have been unset while
	// zoomed)
	if(!$ZoomON) {
		if(isObject(kScriptWheelZoomMap))
			kScriptWheelZoomMap.pop();
	}
}

//-----------------------------------------------------------------------------

function setFOV(%fov)
{
	parent::setFOV(%fov);

	if($kPref::rangeReticles) {
		if($kScript::mortarReticleActive)
			kScriptBuildMortarReticle(0);
		if($kScript::grenadeReticleActive)
			kScriptBuildGrenadeReticle();
	}
}

//-----------------------------------------------------------------------------

function ClientCmdSetHudMode(%mode, %type, %node)
{
	parent::clientCmdSetHudMode(%mode, %type, %node);

	switch$($HudMode) {
	case "Standard":
		kScriptBlockDeploySensor(0);
		kScriptBlockReticles(0);
		kScriptRefreshInfantryReticles();
	case "Pilot":
		kScriptBlockDeploySensor(1);
		kScriptBlockReticles(0);
	case "Object":
		// controlling an object always unzooms you
		kScriptUnzoom();
		kScriptBlockDeploySensor(1);
		kScriptDeactivateReticles();
		if($HudModeType $= "MortarBarrelLarge") {
			kScriptBlockReticles(0);
			kScriptBuildMortarReticle(3);
		}
	default:
		kScriptBlockDeploySensor(1);
		kScriptBlockReticles(1);
	}

	kScriptFixFOV();
}

//-----------------------------------------------------------------------------

function GameConnection::onTargetLocked(%con, %state)
{
	parent::onTargetLocked(%con, %state);

	if(!$kPref::autoMissile)
		return;

	$mvTriggerCount0 += ($mvTriggerCount0 & 1) == (%state $= "true") ? 2 : 1;
}

//-----------------------------------------------------------------------------

function GameConnection::onLockWarning(%con, %state)
{
	parent::onLockWarning(%con, %state);

	if(!$kPref::autoFlareLock)
		return;

	if(%state $= "true")
		use("FlareGrenade");
}

//-----------------------------------------------------------------------------

function GameConnection::onHomeWarning(%con, %state)
{
	parent::onHomeWarning(%con, %state);

	if(!$kPref::autoFlareHoming)
		return;

	if(%state $= "true")
		use("FlareGrenade");
}

//-----------------------------------------------------------------------------

function ammoHud::setVisible(%this, %on)
{
	parent::setVisible(%this, %on);

	kScriptBlockReticles(!%on);
}

//-----------------------------------------------------------------------------

function clientCmdSetWeaponsHudActive(%slot, %ret, %vis)
{
	parent::clientCmdSetWeaponsHudActive(%slot, %ret, %vis);

	if(%slot == -1)
		kScriptDeactivateReticles();
	else
		switch$($WeaponNames[%slot]) {
		case "GrenadeLauncher":
			kScriptBuildGrenadeReticle();
		case "Mortar":
			kScriptBuildMortarReticle(1);
		}
}

//-----------------------------------------------------------------------------

function clientCmdSetVWeaponsHudActive(%num, %vType)
{
	parent::clientCmdSetVWeaponsHudActive(%num, %vType);

	if(%vType $= "AssaultVehicle") {
		if(%num == 2)
			kScriptBuildMortarReticle(2);
		else
			kScriptDeactivateReticles();
	}
}

//-----------------------------------------------------------------------------

function clientCmdSetBackpackHudItem(%num, %addItem)
{
	parent::clientCmdSetBackpackHudItem(%num, %addItem);

	if(%addItem)
		kScriptCheckDeployable();
}

//-----------------------------------------------------------------------------

function clientCmdSetStationKeys(%inStation)
{
	parent::clientCmdSetStationKeys(%inStation);

	if(!%inStation)
		kScriptCheckDeployable();
}

//-----------------------------------------------------------------------------

function clientCmdUpdatePackText(%num)
{
	parent::clientCmdUpdatePackText(%num);

	if(%num > 0)
		kScriptCheckDeployable();
}

};
activatePackage(kScriptOverrides);

//*****************************************************************************
// MORTAR/GRENADE RETICLES
//*****************************************************************************

$kScript::mortarTangent[0] = 0.1227;
$kScript::mortarTangent[1] = 0.1547;
$kScript::mortarTangent[2] = 0.1877;
$kScript::mortarTangent[3] = 0.2220;
$kScript::mortarTangent[4] = 0.2578;
$kScript::mortarTangent[5] = 0.2958;
$kScript::mortarTangent[6] = 0.3364;
$kScript::mortarTangent[7] = 0.3806;
$kScript::mortarTangent[8] = 0.4296;
$kScript::mortarTangent[9] = 0.4854;
$kScript::mortarTangent[10] = 0.5520;
$kScript::mortarTangent[11] = 0.6376;
$kScript::mortarTangent[12] = 0.7708;

$kScript::mortarTangentClassic[0] = 0.1283;
$kScript::mortarTangentClassic[1] = 0.1619;
$kScript::mortarTangentClassic[2] = 0.1966;
$kScript::mortarTangentClassic[3] = 0.2328;
$kScript::mortarTangentClassic[4] = 0.2709;
$kScript::mortarTangentClassic[5] = 0.3115;
$kScript::mortarTangentClassic[6] = 0.3553;
$kScript::mortarTangentClassic[7] = 0.4036;
$kScript::mortarTangentClassic[8] = 0.4581;
$kScript::mortarTangentClassic[9] = 0.5219;
$kScript::mortarTangentClassic[10] = 0.6016;
$kScript::mortarTangentClassic[11] = 0.7157;
$kScript::mortarTangentClassic[12] = 1.0000;  // actually only reaches 396

$kScript::tankMortarTangent[0] = 0.1177;
$kScript::tankMortarTangent[1] = 0.1483;
$kScript::tankMortarTangent[2] = 0.1798;
$kScript::tankMortarTangent[3] = 0.2123;
$kScript::tankMortarTangent[4] = 0.2463;
$kScript::tankMortarTangent[5] = 0.2820;
$kScript::tankMortarTangent[6] = 0.3199;
$kScript::tankMortarTangent[7] = 0.3608;
$kScript::tankMortarTangent[8] = 0.4056;
$kScript::tankMortarTangent[9] = 0.4556;
$kScript::tankMortarTangent[10] = 0.5135;
$kScript::tankMortarTangent[11] = 0.5837;
$kScript::tankMortarTangent[12] = 0.6776;
$kScript::tankMortarTangent[13] = 0.8493;

$kScript::tankMortarTangentClassic[0] = 0.1608;
$kScript::tankMortarTangentClassic[1] = 0.2041;
$kScript::tankMortarTangentClassic[2] = 0.2498;
$kScript::tankMortarTangentClassic[3] = 0.2988;
$kScript::tankMortarTangentClassic[4] = 0.3524;
$kScript::tankMortarTangentClassic[5] = 0.4127;
$kScript::tankMortarTangentClassic[6] = 0.4834;
$kScript::tankMortarTangentClassic[7] = 0.5720;
$kScript::tankMortarTangentClassic[8] = 0.7017;

$kScript::turretMortarTangent[0] = 0.1306;
$kScript::turretMortarTangent[1] = 0.2004;
$kScript::turretMortarTangent[2] = 0.2765;
$kScript::turretMortarTangent[3] = 0.3635;
$kScript::turretMortarTangent[4] = 0.4706;
$kScript::turretMortarTangent[5] = 0.6251;

$kScript::turretMortarTangentClassic[0] = 0.1530;
$kScript::turretMortarTangentClassic[1] = 0.1939;
$kScript::turretMortarTangentClassic[2] = 0.2369;
$kScript::turretMortarTangentClassic[3] = 0.2825;
$kScript::turretMortarTangentClassic[4] = 0.3320;
$kScript::turretMortarTangentClassic[5] = 0.3867;
$kScript::turretMortarTangentClassic[6] = 0.4492;
$kScript::turretMortarTangentClassic[7] = 0.5241;
$kScript::turretMortarTangentClassic[8] = 0.6222;
$kScript::turretMortarTangentClassic[9] = 0.7866;

$kScript::grenadeTangent[0] = 0.1124;
$kScript::grenadeTangent[1] = 0.1714;
$kScript::grenadeTangent[2] = 0.2342;
$kScript::grenadeTangent[3] = 0.3030;
$kScript::grenadeTangent[4] = 0.3816;
$kScript::grenadeTangent[5] = 0.4770;
$kScript::grenadeTangent[6] = 0.6086;
$kScript::grenadeTangent[7] = 0.9610;

$kScript::grenadeTangentClassic[0] = 0.1133;
$kScript::grenadeTangentClassic[1] = 0.1728;
$kScript::grenadeTangentClassic[2] = 0.2361;
$kScript::grenadeTangentClassic[3] = 0.3057;
$kScript::grenadeTangentClassic[4] = 0.3853;
$kScript::grenadeTangentClassic[5] = 0.4826;
$kScript::grenadeTangentClassic[6] = 0.6184;
$kScript::grenadeTangentClassic[7] = 1.0000;  // actually only reaches 224

//-----------------------------------------------------------------------------

function kScriptBuildMortarReticle(%mortarType)
{
	if(!$kPref::rangeReticles)
		return;

	kScriptMortarReticle.position = getword($pref::Video::resolution, 0)/2-16 @ " " @ getword($pref::Video::resolution, 1)/2-1;
	kScriptMortarReticle.extent = "63 " @ getword($pref::Video::resolution, 1)/2+1;

	%scaleFactor = 0.5 * getword($pref::Video::resolution, 0) /
	 mTan(($ZoomOn?$pref::player::currentFOV:$pref::player::defaultFOV) * 0.0087266);
	
	if(%mortarType == 0)
		%mortarType = $kScript::lastMortarType;
	$kScript::lastMortarType = %mortarType;
	
	%classic = (getRecord($ServerInfo, 2) $= "Classic") ? 1 : 0;

	switch(%mortarType) {
	case 1:
		// infantry mortar
		%tickOffset = 2;
		%numTicks = 12;
		%minRange = 100;
		%maxRange = 400;
	case 2:
		// tank mortar
		%tickOffset = 2;
		%numTicks = %classic ? 8 : 13;
		%minRange = 100;
		%maxRange = %classic ? 300 : 425;
	case 3:
		// turret mortar
		%tickOffset = %classic ? 2 : 0;
		%numTicks = %classic ? 9 : 5;
		%minRange = %classic ? 100 : 50;
		%maxRange = %classic ? 325 : 175;
	}
	
	for(%i = 0; %i <= %numTicks; %i++) {
		switch(%mortarType) {
		case 1:
			%currentTangent = %classic ? $kScript::mortarTangentClassic[%i] : $kScript::mortarTangent[%i];
		case 2:
			%currentTangent = %classic ? $kScript::tankMortarTangentClassic[%i] : $kScript::tankMortarTangent[%i];
		case 3:
			%currentTangent = %classic ? $kScript::turretMortarTangentClassic[%i] : $kScript::turretMortarTangent[%i];
		}
		%currentY = mFloor(%currentTangent * %scaleFactor + 0.5);
		(KSR_mortarTick @ (%i + %tickOffset)).position = "0 " @ %currentY-4;
		if(!(%i & 1) || %i == 13)
			(KSR_mortarRange @ (%i + %tickOffset)).position = "32 " @ %currentY-5;
	}

	KSR_mortarTick0.setVisible(%minRange <= 50);
	KSR_mortarRange0.setVisible(%minRange <= 50);
	KSR_mortarTick1.setVisible(%minRange <= 75);

	// always show 100-175
	
	KSR_mortarTick6.setVisible(%maxRange >= 200);
	KSR_mortarRange6.setVisible(%maxRange >= 200);
	KSR_mortarTick7.setVisible(%maxRange >= 225);
	KSR_mortarTick8.setVisible(%maxRange >= 250);
	KSR_mortarRange8.setVisible(%maxRange >= 250);
	KSR_mortarTick9.setVisible(%maxRange >= 275);
	KSR_mortarTick10.setVisible(%maxRange >= 300);
	KSR_mortarRange10.setVisible(%maxRange >= 300);
	KSR_mortarTick11.setVisible(%maxRange >= 325);
	KSR_mortarTick12.setVisible(%maxRange >= 350);
	KSR_mortarRange12.setVisible(%maxRange >= 350);
	KSR_mortarTick13.setVisible(%maxRange >= 375);
	KSR_mortarTick14.setVisible(%maxRange >= 400);
	KSR_mortarRange14.setVisible(%maxRange >= 400);
	KSR_mortarTick15.setVisible(%maxRange >= 425);
	KSR_mortarRange15.setVisible(%maxRange >= 425);

	$kScript::mortarReticleActive = 1;
	if(!$kScript::blockReticles)
		kScriptMortarReticle.setVisible(1);
}

//-----------------------------------------------------------------------------

function kScriptBuildGrenadeReticle()
{
	if(!$kPref::rangeReticles)
		return;

	kScriptGrenadeReticle.position = getword($pref::Video::resolution, 0)/2-16 @ " " @ getword($pref::Video::resolution, 1)/2-1;
	kScriptGrenadeReticle.extent = "63 " @ getword($pref::Video::resolution, 1)/2+1;

	%scaleFactor = 0.5 * getword($pref::Video::resolution, 0) /
	 mTan(($ZoomOn?$pref::player::currentFOV:$pref::player::defaultFOV) * 0.0087266);
	
	%classic = (getRecord($ServerInfo, 2) $= "Classic") ? 1 : 0;

	for(%i = 0; %i <= 7; %i++) {
		%currentTangent = %classic ? $kScript::grenadeTangentClassic[%i] : $kScript::grenadeTangent[%i];
		%currentY = mFloor(%currentTangent * %scaleFactor + 0.5);
		(KSR_grenadeTick @ %i).position = "0 " @ %currentY-4;
		if(!(%i & 1) || %i == 7)
			(KSR_grenadeRange @ %i).position = "32 " @ %currentY-5;
	}

	$kScript::grenadeReticleActive = 1;
	if(!$kScript::blockReticles)
		kScriptGrenadeReticle.setVisible(1);
}

//-----------------------------------------------------------------------------

function kScriptDeactivateReticles()
{
	kScriptMortarReticle.setVisible(0);
	kScriptGrenadeReticle.setVisible(0);
	$kScript::mortarReticleActive = 0;
	$kScript::grenadeReticleActive = 0;
}

//-----------------------------------------------------------------------------

function kScriptBlockReticles(%block)
{
	if(%block) {
		kScriptMortarReticle.setVisible(0);
		kScriptGrenadeReticle.setVisible(0);
		$kScript::blockReticles = 1;
	}
	else {
		if($kScript::blockReticles && $kScript::mortarReticleActive)
			kScriptMortarReticle.setVisible(1);
		if($kScript::blockReticles && $kScript::grenadeReticleActive)
			kScriptGrenadeReticle.setVisible(1);
		$kScript::blockReticles = 0;
	}
}

//-----------------------------------------------------------------------------

function kScriptRefreshInfantryReticles()
{
	kScriptDeactivateReticles();

	switch$(loadout.getCurrentWeapon()) {
	case "GrenadeLauncher":
		kScriptBuildGrenadeReticle();
	case "Mortar":
		kScriptBuildMortarReticle(1);
	}
}

//*****************************************************************************
// JOYSTICK VEHICLE CONTROL AND VEHICLE CONTROL OVERRIDES
//*****************************************************************************

function kScriptVehicleStart()
{
	$kScript::pilotingVehicle = 1;
	$kScript::joystickWasEnabled = $pref::Input::JoystickEnabled;

	// apply vehicle control overrides
	kScriptVehicleMap.push();
	
	if(!$kPref::vehicleJoy)
		return;

	enableJoystick();
	
	passengerKeys.bind(mouse0, "xaxis", S, moveMap.getScale(mouse0, "xaxis"), kScriptVehicleLookYaw);
	passengerKeys.bind(mouse0, "yaxis", moveMap.isInverted(mouse0, "yaxis")?"SI":"S", moveMap.getScale(mouse0, "yaxis"), kScriptVehicleLookPitch);
	passengerKeys.bind(joystick0, "xaxis", SD, "-0.2 0.2", 1, kScriptVehicleYaw);
	passengerKeys.bind(joystick0, "yaxis", SDI, "-0.2 0.2", 1, kScriptVehiclePitch);

	$mvForwardAction = 0;
	$mvBackwardAction = 0;
	$mvLeftAction = 0;
	$mvRightAction = 0;
	$mvYawRightSpeed = 0;
	$mvYawLeftSpeed = 0;
	$mvPitchDownSpeed = 0;
	$mvPitchUpSpeed = 0;
	$kScript::appliedYaw = 0;
	$kScript::appliedPitch = 0;
	$kScript::lastYawTime = GetSimTime();
	$kScript::lastPitchTime = GetSimTime();
}

//-----------------------------------------------------------------------------

function kScriptVehicleEnd()
{
	if($kScript::pilotingVehicle && !$kScript::joystickWasEnabled) {
		// about to disable joystick, so release any triggers which are
		// currently active due to joystick buttons
		if(kScriptIsJoystickBind("mouseFire"))
			mouseFire(0);
		if(kScriptIsJoystickBind("altTrigger"))
			altTrigger(0);
		if(kScriptIsJoystickBind("jump"))
			jump(0);
		if(kScriptIsJoystickBind("mouseJet"))
			mouseJet(0);
		if(kScriptIsJoystickBind("throwGrenade"))
			throwGrenade(0);
		if(kScriptIsJoystickBind("placeMine"))
			placeMine(0);

		disableJoystick();
	}

	$kScript::pilotingVehicle = 0;

	// undo vehicle control overrides
	kScriptVehicleMap.pop();

	if(!$kPref::vehicleJoy)
		return;

	$mvForwardAction = 0;
	$mvBackwardAction = 0;
	$mvLeftAction = 0;
	$mvRightAction = 0;
	
	$mvFreeLook = 0;
}

//-----------------------------------------------------------------------------

function kScriptVehicleAirSettings(%vType, %node)
{
	$kScript::aircraft = 1;
	$kScript::maxSteeringAngle = 3.14;
	if(%vType $= "HAPC")
		$kScript::autoStabilizeSpeed = 10;
	else
		$kScript::autoStabilizeSpeed = 15;
}

//-----------------------------------------------------------------------------

function kScriptVehicleGroundSettings(%vType, %node)
{
	$kScript::aircraft = 0;
	$kScript::maxSteeringAngle = 0.785;
}

//-----------------------------------------------------------------------------

function kScriptVehicleYaw(%val)
{
	// update times
	%now = GetSimTime();
	%elapsed = %now - $kScript::lastYawTime;
	$kScript::lastYawTime = %now;
	
	// account for the input damping of aircraft at slow speeds
	if($kScript::aircraft && getControlObjectSpeed() < $kScript::autoStabilizeSpeed)
		$kScript::appliedYaw *= (1 - 0.05 * (%elapsed/33.3));

	// determine the new applied yaw  
	%newAppliedYaw = %val * $kScript::maxSteeringAngle;

	// play nice with lookyaw/lookpitch and don't apply control unless it's necessary
	if(abs(%newAppliedYaw - $kScript::appliedYaw)/$kScript::maxSteeringAngle > 0.01)
	{
		$mvFreeLook = 0;

		// move yaw from old applied to new applied
		$mvYaw += %newAppliedYaw - $kScript::appliedYaw;
		$kScript::appliedYaw = %newAppliedYaw;
	}

	// sweep away any errors if we're turning as hard as possible
	if(abs(%val) == 1) {
		$mvFreeLook = 0;
		$mvYaw += %val * 0.1;
	}
}

//-----------------------------------------------------------------------------

function kScriptVehiclePitch(%val)
{
	// update times
	%now = GetSimTime();
	%elapsed = %now - $kScript::lastPitchTime;
	$kScript::lastPitchTime = %now;

	// account for the input damping of aircraft at slow speeds
	if($kScript::aircraft && getControlObjectSpeed() < $kScript::autoStabilizeSpeed)
		$kScript::appliedPitch *= (1 - 0.05 * (%elapsed/33.3));

	// figure the new applied pitch  
	%newAppliedPitch = %val * $kScript::maxSteeringAngle;

	// play nice with lookpitch/lookpitch and don't apply control unless it's necessary
	if(abs(%newAppliedPitch - $kScript::appliedPitch)/$kScript::maxSteeringAngle > 0.01)
	{
		$mvFreeLook = 0;

		// move pitch from old applied to new applied
		$mvPitch += %newAppliedPitch - $kScript::appliedPitch;
		$kScript::appliedPitch = %newAppliedPitch;
	}

	// sweep away any accumulated if we're turning as hard as possible
	if(abs(%val) == 1) {
		$mvFreeLook = 0;
		$mvPitch += %val * 0.1;
	}
}

//-----------------------------------------------------------------------------

function kScriptVehicleLookYaw(%val)
{
	// don't try to freelook when the vehicle is turning
	if(%val
		&& abs($kScript::appliedPitch)/$kScript::maxSteeringAngle < 0.1
		&& abs($kScript::appliedYaw)/$kScript::maxSteeringAngle < 0.1)
	{
		$mvFreeLook = 1;
		$mvYaw += getMouseAdjustAmount(%val);
	}
}

//-----------------------------------------------------------------------------

function kScriptVehicleLookPitch(%val)
{
	// don't try to freelook when the vehicle is turning
	if(%val
		&& abs($kScript::appliedPitch)/$kScript::maxSteeringAngle < 0.1
		&& abs($kScript::appliedYaw)/$kScript::maxSteeringAngle < 0.1)
	{
		$mvFreeLook = 1;
		$mvPitch += getMouseAdjustAmount(%val);
	}
}

//-----------------------------------------------------------------------------

function kScriptIsJoystickBind(%cmd)
{
	// find the active binding for this command, assuming we're currently
	// piloting a vehicle
	%temp = kScriptVehicleMap.getBinding(%cmd);
	if(%temp $= "")
		%temp = passengerKeys.getBinding(%cmd);

	// see if the device includes "joystick"
	%device = getField(%temp, 0);
	if(strstr(%device, "joystick") != -1)
		return 1;
	else
		return 0;
}

//*****************************************************************************
// MISC
//*****************************************************************************

function kScriptBlockDeploySensor(%block)
{
	if(!$kPref::deployableIndicator)
		return;

	if(%block)
		deploySensor.setVisible(0);
	else
		kScriptCheckDeployable(1);
}

//-----------------------------------------------------------------------------

function kScriptCheckDeployable(%show)
{
	if(!$kPref::deployableIndicator)
		return;

	if(strStr(loadout.getPack(), "Deployable") != -1) {
		useBackpack(1);
		if(%show)
			deploySensor.setVisible(1);
	}
}

//-----------------------------------------------------------------------------

function kScriptStartBombSight()
{
	if(!$kPref::pilotBombSight)
		return;

	clientCmdStartBomberSight();
}

//-----------------------------------------------------------------------------

function kScriptEndBombSight()
{
	if(!$kPref::pilotBombSight)
		return;

    clientCmdEndBomberSight();
}

//-----------------------------------------------------------------------------

function kScriptUnblockGrenade()
{
	$kScript::blockGrenade = 0;
}

//-----------------------------------------------------------------------------

function kScriptPrimeGrenade()
{
	if($kPref::maxThrow) {
		commandToServer('startThrowCount');
		if(!($mvTriggerCount4 & 1))
			$mvTriggerCount4++;
	}
}

//-----------------------------------------------------------------------------

function kScriptUnzoom()
{
	if($ZoomOn) {
		// try to unzoom
		toggleZoom(0);
		if($ZoomOn)
			// looks like the player has a zoom toggle, so try again
			toggleZoom(1);
	}
}

//-----------------------------------------------------------------------------

function kScriptFixFOV()
{
	// make sure fov is correct if we're unzoomed... T2 has a nasty habit
	// of resetting fov to 90 instead of $pref::player::defaultFOV.
	schedule(200, 0, kScriptCheckUnzoom);
	schedule(600, 0, kScriptCheckUnzoom); // safety
}

//-----------------------------------------------------------------------------

function kScriptCheckUnzoom()
{
	if(!$ZoomOn)
		setFOV($pref::player::defaultFOV);
}

//-----------------------------------------------------------------------------

function kScriptZoomAxis(%inc)
{
	// assume mousewheel with increments of 120
	%inc = mCeil(%inc / 120);
	for(%i = 0; %i < %inc; %i++)
		$pref::player::currentFOV /= $kPref::wheelZoomSens;
	for(%i = 0; %i > %inc; %i--)
		$pref::player::currentFOV *= $kPref::wheelZoomSens;

	// cap it
	if($pref::player::currentFOV > 50)
		$pref::player::currentFOV = 50;
	else if($pref::player::currentFOV < 5)
		$pref::player::currentFOV = 5;

	// set new fov if zoomed
	if($ZoomOn)
		setFOV($pref::player::currentFOV);
}

//-----------------------------------------------------------------------------

function kScriptRangefinder(%keydown)
{
	if(%keydown) {
		useTargetingLaser(1);
		if(!($mvTriggerCount0 & 1))
			$mvTriggerCount0++;
		schedule(1000, 0, kScriptStopRangefinder);
	}
}

//-----------------------------------------------------------------------------

function kScriptStopRangefinder()
{
	kScriptUnzoom();
	
	if($mvTriggerCount0 & 1)
		$mvTriggerCount0++;
	
	loadout.useWeapon(loadout.getPreviousWeapon());
}

//-----------------------------------------------------------------------------

function kScriptJumpJet(%keydown)
{
	mouseJet(%keydown);

	if(%keydown && !($mvTriggerCount2 & 1)) {
		jump(1);
		schedule(100, 0, jump, 0);
	}
}

//*****************************************************************************
// OPTIONS DIALOG SUPPORT
//*****************************************************************************

function kScriptSavePrefs(%this)
{
	export("$kPref::*", "prefs/kPrefs.cs", false);
	kScriptVehicleMap.save("prefs/kPrefs.cs", true);
}

//-----------------------------------------------------------------------------

function kScriptBuildFullMapString(%index)
{
	%name       = $RemapName[%index];
	%cmd        = $RemapCmd[%index];

	%temp = kScriptVehicleMap.getBinding(%cmd);
	%device = getField(%temp, 0);
	%object = getField(%temp, 1);

	if(%device !$= "" && %object !$= "")
		%mapString = getMapDisplayName(%device, %object);
	else
		%mapString = "";

	return(%name TAB %mapString);
}

//-----------------------------------------------------------------------------

function kScriptChangeVehicleBind(%device, %action, %oldIndex, %newIndex)
{
	kScriptVehicleMap.bind(%device, %action, $RemapCmd[%newIndex]);

	KSO_vehicleControls.setRowById(%oldIndex, kScriptBuildFullMapString(%oldIndex));
	KSO_vehicleControls.setRowById(%newIndex, kScriptBuildFullMapString(%newIndex));
}

//------------------------------------------------------------------------------

function kScriptOptions::onWake(%this)
{
	// most scripts that register new commands in the control list tend to chain
	// off of OptionsDlg::onWake, so this is a hack to get those registered
	// before showing the vehicle control overrides screen.
	OptionsDlg::onWake();

	KSO_GeneralPane.setVisible(false); 
	KSO_ZoomPane.setVisible(false);
	KSO_ControlsPane.setVisible(false);

	KSO_GeneralTab.setValue(false); 
	KSO_ZoomTab.setValue(false);
	KSO_ControlsTab.setValue(false);

	KSO_wheelZoomSens.setValue($kPref::wheelZoomSens);
	KSO_defZoomFOV.setValue($kPref::defaultZoomFOV);
	KSO_unzoomedFOV.setValue($pref::player::defaultFOV);

	KSO_vehicleControls.fillList();

	%this.setPane(%this.pane);
}

//-----------------------------------------------------------------------------

function kScriptOptions::setPane(%this, %pane)
{
	// deactivate current pane
	%paneCtrl = "KSO_" @ %this.pane @ "Pane";
	%paneCtrl.setVisible(false);

	%tabCtrl = "KSO_" @ %this.pane @ "Tab";
	%tabCtrl.setValue(false);
  
	// activate new one
	%paneCtrl = "KSO_" @ %pane @ "Pane";
	%paneCtrl.setVisible(true);

	%tabCtrl = "KSO_" @ %pane @ "Tab";
	%tabCtrl.setValue(true);

	%this.pane = %pane;
}

//-----------------------------------------------------------------------------

function kScriptOptionsRemapDlg::onWake(%this)
{
	$enableDirectInput = "1";
	activateDirectInput();
}

//-----------------------------------------------------------------------------

function kScriptOptionsRemapDlg::onSleep(%this)
{
	$enableDirectInput = "1";
	deactivateDirectInput();
}

//-----------------------------------------------------------------------------

function KSO_vehicleControls::fillList(%this)
{
   %this.clear();
   for(%i = 0; %i < $RemapCount; %i++)
      %this.addRow(%i, kScriptBuildFullMapString(%i));
}

//-----------------------------------------------------------------------------

function KSO_vehicleControls::onDeleteKey(%this, %index)
{
	clearMapping(kScriptVehicleMap, $RemapCmd[%index]);
	%this.setRowById(%index, kScriptBuildFullMapString(%index));
}

//-----------------------------------------------------------------------------

function KSO_vehicleControls::doRemap(%this)
{
	%index = %this.getSelectedId();
	%name = $RemapName[%index];

	KSO_remapFrame.setTitle("REMAP \"" @ %name @ "\"");
	KSO_remapInputCtrl.index = %index;
	Canvas.pushDialog(kScriptOptionsRemapDlg);
}

//-----------------------------------------------------------------------------

function KSO_remapInputCtrl::onInputEvent(%this, %device, %action)
{
   Canvas.popDialog(kScriptOptionsRemapDlg);

   // do nothing if key was escape
	if(%device $= "keyboard" && %action $= "escape")
      return;
   
   %cmd = $RemapCmd[%this.index];
   %name = $RemapName[%this.index];

   // find the previous command for this key
   %prevCmd = kScriptVehicleMap.getCommand(%device, %action);

	// if it's the same as the current command, do nothing
	if(%prevCmd $= %cmd)
		return;

	// if there was no previous command, go ahead and bind
	if(%prevCmd $= "") {
		kScriptVehicleMap.bind(%device, %action, %cmd);
		KSO_vehicleControls.setRowById(%this.index, kScriptBuildFullMapString(%this.index));
	}

	// otherwise, give the standard warning/rejection
	else {
      %mapName = getMapDisplayName(%device, %action);
      %prevCmdIndex = findRemapCmdIndex(%prevCmd);
      
		if(%prevCmdIndex == -1)
         MessageBoxOK("REMAP FAILED", "\"" @ %mapName @ "\" is already bound to a non-remappable command!");

      else {
         %prevCmdName = $RemapName[%prevCmdIndex];

         MessageBoxYesNo("WARNING",
            "\"" @ %mapName @ "\" is already bound to \"" @ %prevCmdName @	"\"!\nDo you want to undo this mapping?",
            "kScriptChangeVehicleBind(" @ %device @ ", \"" @ %action @ "\", " @ %prevCmdIndex @ ", " @ %this.index @ ");",
			"");
      }
   }
}
