// #name = Loadout Support
// #version = 0.0.6
// #date = August 8, 2001
// #author = Daniel Neilsen (aka Wizard_TPG)
// #email = wizardsworld@bigpond.com
// #web = http://www.tribalwar.com/wizard/
// #description = Determines players current loadout status from HUD information
// #status = release
// #include = support/callback.cs
// ---------------------------------------------------------------------------

// note from Kerb:  loadout.cs is normally part of support.vl2, but the latest
// T2 patch breaks it.  I'm including the fixed version directly in my script
// pack until an official fixed version is available.

//
//	Usage Notes and Examples:
//
// 	Callbacks included in this support script:
//		PlayerSpawn - called when player spawns.
//		WeaponChange - When player changes weapon.  Returns new weapon as 1st variable.
//		PlayerUseInv - Player used the inventory station
//		PlayerDeath - Player died somehow.  Could have changed teams or gone to obs.
//		MineUsed - A mine was removed from the players inventory either by use or otherwise
//		MineReceived - A mine was added to the players inventory
//		RepairKitUsed - A Repairkit was removed from the players inventory.
//		RepairKitReceived - A Repairkit was added to the players inventory.
//		BeaconUsed - A Beacon was removed from the players inventory.
//		BeaconReceived - A Beacon was added to the players inventory.
//		GrenadeUsed - A Grenade was removed from the players inventory.
//		GrenadeReceived - A Grenade was added to the players inventory.
//		ModTypeChange - When you enter a server this callback returns mod name
//
//
//	Useful Functions in this support script:
//		loadout.isLoadoutWeapon(%name); - Will return true or false if weapon is in players inv
//		loadout.getCurrentWeapon(); - Returns the name of the current weapon in players hand. "" for none.
//		loadout.getPreviousWeapon(); - Returns the name of the previously used weapon.  "" for none
//		loadout.getWeaponAmmo(%name); - Returns the ammo amount for that weapon in players inv. -1 for infinite.
//		loadout.getPack(); - Returns name of current pack.  "" for none.
//		loadout.getMineAmmo(); - Returns number of mines in players inv.
//		loadout.getRepairKit(); - Returns number of repairkits in players inv. (ie. 1 or 0)
//		loadout.getBeaconAmmo(); - Returns number of beacons in players inv.
//		loadout.getGrenadeAmmo(); - Returns number of grenades in players inv.
//		loadout.UseWeapon(%weapon); - Input the weapon name to load the correct weapon function
//		loadout.getGrenadeType(); - Returns the type of grenade in players inv.
//		loadout.getArmorType(); - Returns current type of armor (if mod with support).
//		loadout.getModType(); - Returns current server mod type.
//		NOTE: getGrenadeType can not currently be determined due to lack of HUD data and will always return "Grenade"
//
//
//	Possible WeaponTypes Are:
//		Blaster
//		Plasma
//		Chaingun
//		Disc
//		GrenadeLauncher
//		SniperRifle
//		ELFGun
//		Mortar
//		MissileLauncher
//		ShockLance
//		TargetingLaser
//		RepairGun
//		ParticleGun			-	Shifter
//		HeaterGun			-	Shifter
//		RailGun				-	Shifter
//		Flamer				-	Shifter
//		EngineerRepairGun	-	Shifter
//		GravitronGun		-	Shifter
//		VoltProjector		-	Shifter
//		BoomStick			-	Shifter
//
//		Note: All weapons from any mod should be fine
//
//
//
//	Possible Grenade Types Are:
//		Grenade
//		FlashGrenade
//		ConcussionGrenade
//		FlareGrenade
//		CameraGrenade
//
//
//	Possible Mine Types Are:
//		Mine
//
//
//	Possible Pack Types Are:
//		AmmoPack
//		CloakingPack`
//		EnergyPack
//		RepairPack
//		SatchelCharge
//		ShieldPack
//		InventoryDeployable
//		MotionSensorDeployable
//		PulseSensorDeployable
//		TurretOutdoorDeployable
//		TurretIndoorDeployable
//		SensorJammerPack
//		AABarrelPack
//		FusionBarrelPack
//		MissileBarrelPack
//		PlasmaBarrelPack
//		ELFBarrelPack
//		MortarBarrelPack
//		ThrusterPack 					-	TAC2
//		MorphPack						-	Shifter
//		TelePack						-	Shifter
//		DetPack							-	Shifter
//		DeployableThumperPack			-	Shifter
//		heavydevistatorcannon			-	Shifter
//		heavyplasmacannon				-	Shifter
//		FFBeacon						-	Shifter
//		DeployableTurretPack			-	Shifter
//		DeployableLaserBarrelPack		-	Shifter
//		DeployableShockerBarrelPack		-	Shifter
//		CycloneLauncherPack				-	Shifter
//		ShieldBeacon					-	Shifter
//		FFCube							-	Shifter
//		JammerBeacon					-	Shifter
//		ForceFieldDeployable			-	Shifter
//		RepairBeaconDeployable			-	Shifter
//
//
//
//
//	Note:   This script SHOULD work for mods as well.
//			I have only added mod weapon/pack names in here for TAC2 and Shifter although
//			other mods should also be fine.
//
//
//============================================================================

package Inventory_Support
{
	function clientCmdSetWeaponsHudItem(%slot, %ammoAmount, %addItem)
	{
	   if(%addItem)
			loadout::addweapon(%this, %slot);
	   else
		  	loadout::removeweapon(%this, %slot);
	   Parent::clientCmdSetWeaponsHudItem(%slot, %ammoAmount, %addItem);
	}


	function clientCmdSetWeaponsHudAmmo(%slot, %ammoAmount)
	{
	   	loadout.setWeaponAmmo(%slot, %ammoAmount);
	   	Parent::clientCmdSetWeaponsHudAmmo(%slot, %ammoAmount);
	}

	function clientCmdSetWeaponsHudActive(%slot, %ret, %vis)
	{
	   loadout.setCurrentWeapon(%slot);
	   Parent::clientCmdSetWeaponsHudActive(%slot, %ret, %vis);
	}

	function clientCmdSetRepairReticle()
	{
	   loadout.setCurrentWeapon(0, "RepairGun");
	   Parent::clientCmdSetRepairReticle();
	}

	function clientCmdSetWeaponsHudClearAll()
	{
	   	loadout.clearall();
	   	Parent::clientCmdSetWeaponsHudClearAll();
	}

	//=============================================================

	function clientCmdSetBackpackHudItem(%num, %addItem)
	{
		if(%addItem)
			loadout.addpack(%num);
		else
			loadout.clearpack();
		Parent::clientCmdSetBackpackHudItem(%num, %addItem);

	}

	//=============================================================

	function clientCmdSetInventoryHudItem(%slot, %amount, %addItem)
	{
	   callback.trigger(PlayerSpawn);
	   Parent::clientCmdSetInventoryHudItem(%slot, %amount, %addItem);
	}

	function clientCmdSetInventoryHudAmount(%slot, %amount)
	{
	   loadout.setInvData(%slot, %amount);
	   Parent::clientCmdSetInventoryHudAmount(%slot, %amount);
	}

	function clientCmdSetInventoryHudClearAll()
	{
	   loadout.PlayerDeath();
	   Parent::clientCmdSetInventoryHudClearAll();
	}

	function clientCmdSetArmorType(%type)
	{
	   loadout.SetArmorType(%type);
	   Parent::clientCmdSetArmorType(%type);
	}
};

activatepackage(Inventory_Support);

if(!isObject(loadout))
{
    new ScriptObject(loadout)
    {
        class = loadout;
    };
}

function handleLoadoutHUDMissionInfo(%msgType, %msgString, %missionname, %missiontype, %servername)
{
	loadout.currentServerMod = getRecord( $ServerInfo, 2 );
	callback.trigger(ModTypeChange, loadout.currentServerMod);
}
addMessageCallback( 'MsgMissionDropInfo', handleLoadoutHUDMissionInfo );


//==================================================

function loadout::addweapon(%this, %slot)
{
	%this.playerdead = false;
	%name = $WeaponNames[%slot];
	$Loadout::weapon[%name] = true;
}

function loadout::removeweapon(%this, %slot)
{
	%name = $WeaponNames[%slot];
	$Loadout::weapon[%name] = false;
}

function loadout::clearweapon(%this)
{
	for(%slot=0; %slot<11; %slot++)
	{
		%name = $WeaponNames[%slot];
		$Loadout::weapon[%name] = false;
	}
}

function loadout::setCurrentWeapon(%this, %slot, %other)
{
	%this.playerdead = false;
	if(%this.currentWeapon !$= "")
		%this.previousWeapon = %this.currentWeapon;
	if(%slot == -1)
		%this.currentWeapon = "";
	else if(%other $= "")
		%this.currentWeapon = $WeaponNames[%slot];
	else
		%this.currentWeapon = %other;
	callback.trigger(WeaponChange, %this.currentWeapon);
}

function loadout::setWeaponAmmo(%this, %slot, %ammo)
{
	%name = $WeaponNames[%slot];
	%this.weaponAmmo[%name] = %ammo;
}

function loadout::isLoadoutWeapon(%this, %name)
{
	if(%name $= "")
		return false;

	for(%slot=0; %slot<100; %slot++)
	{
		%namedata = $WeaponNames[%slot];
		if(%namedata $= "")
			return false;
		if(%name $= %namedata)
		{
			%val = $Loadout::weapon[%namedata] == 1 ? 1 : 0;
			return %val;
		}
	}
	return false;
}

function loadout::getCurrentWeapon(%this)
{
	return %this.currentWeapon;
}

function loadout::getPreviousWeapon(%this)
{
	return %this.previousWeapon;
}

function loadout::getWeaponAmmo(%this, %name)
{
	%val = %this.weaponAmmo[%name] $= "" ? -1 : %this.weaponAmmo[%name];
	return %val;
}


//==============================================================

function loadout::addpack(%this, %slot)
{
	%this.pack = $BackpackHudData[%slot, itemDataName];
}

function loadout::clearpack(%this)
{
	%this.pack = "";
}

function loadout::getPack(%this)
{
	return %this.pack;
}

//==============================================================

function loadout::setInvData(%this, %slot, %ammo)
{
	%this.playerdead = false;
	if(%slot $= "")
		return;

	for(%num = 0; %num < $InventoryHudCount; %num++)
	{
		if($InventoryHudData[%num, slot] == %slot)
		{
			%numdata = %num;
			%num = $InventoryHudCount;
		}
	}
	%datatype = $InventoryHudData[%numdata, itemDataName];
	if(%datatype $= Mine)
	{
		if(%this.mineAmmo > %ammo)
			callback.trigger(MineUsed);
		else if(%this.mineAmmo < %ammo)
			callback.trigger(MineReceived);
		%this.mineAmmo = %ammo;
	}
	else if(%datatype $= RepairKit)
	{
		if(%this.repairKit > %ammo)
			callback.trigger(RepairKitUsed);
		else if(%this.repairKit < %ammo)
			callback.trigger(RepairKitReceived);
		%this.repairKit = %ammo;
	}
	else if(%datatype $= Beacon)
	{
		if(%this.repbeaconAmmo > %ammo)
			callback.trigger(BeaconUsed);
		else if(%this.beaconAmmo < %ammo)
			callback.trigger(BeaconReceived);
		%this.beaconAmmo = %ammo;
	}
	else
	{
		if(%ammo >= 1)
		{
			if(%this.grenAmmo > %ammo)
				callback.trigger(GrenadeUsed);
			else if(%this.grenAmmo < %ammo)
				callback.trigger(GrenadeReceived);
			%this.grenType = %datatype;
			%this.grenAmmo = %ammo;
		}
		else if(%datatype $= %this.grenType)
		{
			if(%this.grenAmmo > %ammo)
				callback.trigger(GrenadeUsed);
			else if(%this.grenAmmo < %ammo)
				callback.trigger(GrenadeReceived);
			%this.grenAmmo = %ammo;
		}
	}
}

function loadout::clearInvData(%this)
{
	%this.mineAmmo = 0;
	%this.repairKit = 0;
	%this.beaconAmmo = 0;
	%this.grenType = "";
	%this.grenAmmo = 0;
}

function loadout::getMineAmmo(%this)
{
	return %this.mineAmmo;
}

function loadout::getRepairKit(%this)
{
	return %this.repairKit;
}

function loadout::getBeaconAmmo(%this)
{
	return %this.beaconAmmo;
}

function loadout::getGrenadeType(%this)
{
	return %this.grenType;
}

function loadout::getGrenadeAmmo(%this)
{
	return %this.grenAmmo;
}

//=======================================================

function loadout::clearall(%this)
{
	%this.clearweapon();
	%this.clearpack();
	%this.clearInvData();
	if(!%this.playerdead)
		callback.trigger(PlayerUseInv);
}

function loadout::PlayerDeath(%this)
{
	%this.playerdead = true;
	%this.clearall();
	callback.trigger(PlayerDeath);
}

function loadout::UseWeapon(%this, %weapon)
{
	if(%weapon $= "Blaster")
		useBlaster(true);
	else if(%weapon $= "Plasma")
		usePlasma(true);
	else if(%weapon $= "Chaingun")
		useChaingun(true);
	else if(%weapon $= "Disc")
		useDisc(true);
	else if(%weapon $= "GrenadeLauncher")
		useGrenadeLauncher(true);
	else if(%weapon $= "SniperRifle")
		useSniperRifle(true);
	else if(%weapon $= "ELFGun")
		useELFGun(true);
	else if(%weapon $= "Mortar")
		useMortar(true);
	else if(%weapon $= "MissileLauncher")
		useMissileLauncher(true);
	else if(%weapon $= "TargetingLaser")
		useTargetingLaser(true);
	else if(%weapon $= "ShockLance")
		useShockLance(true);
	else
		use( %weapon );
}


//==========================================
//	Mod Functions

function loadout::getModType(%this)
{
	return %this.currentServerMod;
}
